<?php

namespace Drupal\commerce_sql_sanitize\Commands;

use Consolidation\AnnotatedCommand\CommandData;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drush\Commands\DrushCommands;
use Drush\Drupal\Commands\sql\SanitizePluginInterface;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;

/**
 * Provides a Drush sql sanitization plugin for orders.
 *
 * It overwrites the order email and IP addresses.
 */
class SanitizeOrderCommands extends DrushCommands implements SanitizePluginInterface {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Creates a new SanitizeOrderCommands object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(Connection $database, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct();
    $this->database = $database;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   *
   * Overwrites order email and IP addresses.
   *
   * @hook post-command sql-sanitize
   */
  public function sanitize($result, CommandData $commandData) {
    $options = $commandData->options();
    if (!$this->isEnabled($options['sanitize-commerce-order'])) {
      return;
    }
    $this->database->update('commerce_order')
      ->isNotNull('mail')
      ->fields(['mail' => 'sanitized@local.test'])
      ->execute();
    $this->database->update('commerce_order')
      ->isNotNull('ip_address')
      ->fields(['ip_address' => '127.0.0.1'])
      ->execute();

    $this->entityTypeManager->getStorage('commerce_order')->resetCache();

    $this->logger()->success(dt('Order email and IP addresses sanitized.'));
  }

  /**
   * Provides extra options for sql:sanitize.
   *
   * @hook option sql-sanitize
   */
  public function options(Command $command) {
    if (!$this->entityTypeManager->getDefinition('commerce_order', FALSE)) {
      return;
    }
    $command->addOption(
      'sanitize-commerce-order',
      NULL,
      InputOption::VALUE_OPTIONAL,
      dt('By default, order email and IP addresses are sanitized. Specify <info>no</info> to disable that.')
    );
  }

  /**
   * {@inheritdoc}
   *
   * @hook on-event sql-sanitize-confirms
   */
  public function messages(&$messages, InputInterface $input) {
    if (!$this->isEnabled($input->getOption('sanitize-commerce-order'))) {
      return;
    }
    $messages[] = dt('Sanitize order email and IP addresses.');
  }

  /**
   * Test an option value to see if it is enabled.
   *
   * @param string $value
   *   The option value to test.
   *
   * @return bool
   *   TRUE if the value represents a positive value (anything other than 'no'
   *   or '0').
   */
  protected function isEnabled($value) {
    if (!$this->entityTypeManager->getDefinition('commerce_order', FALSE)) {
      return FALSE;
    }
    return $value != 'no' && $value != '0';
  }

}
